@@ -159,6 +159,41 @@ class MatrixTableResource(BaseResource):
         )
 
 
+class VariantDatasetResource(BaseResource):
+    """
+    A Hail VariantDataset resource.
+
+    :param path: The VariantDataset path (typically ending in .vds)
+    :param import_args: Any sources that are required for the import and need to be kept track of and/or passed to the import_func (e.g. .vcf path for an imported VCF)
+    :param import_func: A function used to import the VariantDataset. `import_func` will be passed the `import_args` dictionary as kwargs.
+    """
+
+    expected_file_extensions: List[str] = [".vds"]
+
+    def vds(self, force_import: bool = False) -> hl.vds.VariantDataset:
+        """
+        Read and return the Hail VariantDataset resource.
+
+        :return: Hail VariantDataset resource
+        """
+        if self.path is None or force_import:
+            return self.import_func(**self.import_args)
+        else:
+            return hl.vds.read_vds(self.path)
+
+    def import_resource(self, overwrite: bool = True, **kwargs) -> None:
+        """
+        Import the VariantDataset resource using its import_func and writes it in its path.
+
+        :param overwrite: If set, existing file(s) will be overwritten
+        :param kwargs: Any other parameters to be passed to hl.vds.VariantDataset.write
+        :return: Nothing
+        """
+        self.import_func(**self.import_args).write(
+            self.path, overwrite=overwrite, **kwargs
+        )
+
+
 class PedigreeResource(BaseResource):
     """
     A pedigree resource.
@@ -183,7 +218,9 @@ class PedigreeResource(BaseResource):
         missing: str = "NA",
     ):
         super().__init__(
-            path=path, import_args=import_args, import_func=import_func,
+            path=path,
+            import_args=import_args,
+            import_func=import_func,
         )
 
         self.quant_pheno = quant_pheno
@@ -291,10 +328,14 @@ class BaseVersionedResource:
         self.versions = versions
 
     def __repr__(self):
-        return "{cls}(default_version={default_version}, versions={{{versions}}})".format(
-            cls=self.__class__.__name__,
-            default_version=self.default_version,
-            versions=", ".join(f'"{k}": {repr(v)}' for k, v in self.versions.items()),
+        return (
+            "{cls}(default_version={default_version}, versions={{{versions}}})".format(
+                cls=self.__class__.__name__,
+                default_version=self.default_version,
+                versions=", ".join(
+                    f'"{k}": {repr(v)}' for k, v in self.versions.items()
+                ),
+            )
         )
 
     def __getattr__(self, name):
@@ -339,6 +380,25 @@ class VersionedMatrixTableResource(BaseVersionedResource):
         super().__init__(default_version, versions)
 
 
+class VersionedVariantDatasetResource(BaseVersionedResource):
+    """
+    Versioned VariantDataset resource.
+
+    The attributes (path, import_args and import_func) of the versioned resource are those of the default version of the resource.
+    In addition, all versions of the resource are stored in the `versions` attribute.
+    :param default_version: The default version of this VariantDataset resource (must to be in the `versions` dict)
+
+    :param versions: A dict of version name -> VariantDatasetResource.
+    """
+
+    resource_class = VariantDatasetResource
+
+    def __init__(
+        self, default_version: str, versions: Dict[str, VariantDatasetResource]
+    ):
+        super().__init__(default_version, versions)
+
+
 class VersionedPedigreeResource(BaseVersionedResource, PedigreeResource):
     """
     Versioned Pedigree resource.
@@ -436,6 +496,12 @@ class GnomadPublicResource(BaseResource, ABC):
         if resource_source == GnomadPublicResourceSource.GOOGLE_CLOUD_PUBLIC_DATASETS:
             return f"gs://gcp-public-data--gnomad{relative_path}"
 
+        if resource_source == GnomadPublicResourceSource.REGISTRY_OF_OPEN_DATA_ON_AWS:
+            return f"s3a://gnomad-public-us-east-1{relative_path}"
+
+        if resource_source == GnomadPublicResourceSource.AZURE_OPEN_DATASETS:
+            return f"wasbs://dataset@datasetgnomad.blob.core.windows.net{relative_path}"
+
         return (
             f"{resource_source.rstrip('/')}{relative_path}"  # pylint: disable=no-member
         )
