@@ -1,18 +1,23 @@
+# noqa: D100
+
+import json
 import logging
 import pprint
 from pprint import pformat
-from typing import Dict, List, Tuple
+from typing import Dict, List, Optional, Tuple
 
 import hail as hl
 import pandas as pd
 import pyspark.sql
 from pyspark.ml import Pipeline
-from pyspark.ml.classification import RandomForestClassifier, json
+from pyspark.ml.classification import RandomForestClassifier
 from pyspark.ml.feature import IndexToString, StringIndexer, VectorAssembler
 from pyspark.sql import SparkSession
 from pyspark.sql.functions import col, udf  # pylint: disable=no-name-in-module
 from pyspark.sql.types import ArrayType, DoubleType
 
+from gnomad.utils.file_utils import file_exists
+
 logging.basicConfig(format="%(levelname)s (%(name)s %(lineno)s): %(message)s")
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
@@ -20,9 +25,9 @@ logger.setLevel(logging.INFO)
 
 def run_rf_test(
     mt: hl.MatrixTable, output: str = "/tmp"
-) -> Tuple[pyspark.ml.PipelineModel, hl.MatrixTable]:
+) -> Tuple[pyspark.ml.PipelineModel, hl.Table]:
     """
-    Runs a dummy test RF on a given MT.
+    Run a dummy test RF on a given MT.
 
     1. Creates row annotations and labels to run model on
     2. Trains a RF pipeline model (including median imputation of missing values in created annotations)
@@ -33,7 +38,6 @@ def run_rf_test(
     :param output: Output files prefix to save the RF model
     :return: RF model and MatrixTable after applying RF model
     """
-
     mt = mt.annotate_rows(
         feature1=hl.rand_bool(0.1),
         feature2=hl.rand_norm(0.0, 1.0),
@@ -54,26 +58,20 @@ def run_rf_test(
         )
 
     f3_before_imputation = f3stats(ht)
-    logger.info(
-        "Feature3 defined values before imputation: {}".format(f3_before_imputation.n)
-    )
-    logger.info("Feature3 median: {}".format(f3_before_imputation.med))
+    logger.info("Feature3 defined values before imputation: %d", f3_before_imputation.n)
+    logger.info("Feature3 median: %f", f3_before_imputation.med)
 
     features_to_impute = ["feature3"]
     quantiles = get_columns_quantiles(ht, features_to_impute, [0.5])
     quantiles = {k: v[0] for k, v in quantiles.items()}
 
-    logger.info(
-        "Features median:\n{}".format(f"{k}: {v}\n" for k, v in quantiles.items())
-    )
+    logger.info("Features median:\n%s", [f"{k}: {v}\n" for k, v in quantiles.items()])
     ht = ht.annotate(**{f: hl.or_else(ht[f], quantiles[f]) for f in features_to_impute})
     ht = ht.annotate_globals(medians=quantiles)
 
     f3_after_imputation = f3stats(ht)
-    logger.info(
-        "Feature3 defined values after imputation: {}".format(f3_after_imputation.n)
-    )
-    logger.info("Feature3 median: {}".format(f3_after_imputation.med))
+    logger.info("Feature3 defined values after imputation: %d", f3_after_imputation.n)
+    logger.info("Feature3 median: %f", f3_after_imputation.med)
 
     ht = ht.select("label", "feature1", "feature2", "feature3")
 
@@ -89,13 +87,12 @@ def run_rf_test(
 
 def check_ht_fields_for_spark(ht: hl.Table, fields: List[str]) -> None:
     """
-    Checks specified fields of a hail table for Spark DataFrame conversion (type and name)
+    Check specified fields of a hail table for Spark DataFrame conversion (type and name).
 
     :param ht: input Table
     :param fields: Fields to test
     :return: None
     """
-
     allowed_types = [
         hl.tfloat,
         hl.tfloat32,
@@ -131,7 +128,7 @@ def get_columns_quantiles(
     ht: hl.Table, fields: List[str], quantiles: List[float], relative_error: int = 0.001
 ) -> Dict[str, List[float]]:
     """
-    Computes approximate quantiles of specified numeric fields from non-missing values. Non-numeric fields are ignored.
+    Compute approximate quantiles of specified numeric fields from non-missing values. Non-numeric fields are ignored.
 
     This function returns a Dict of column name -> list of quantiles in the same order specified.
     If a column only has NAs, None is returned.
@@ -142,14 +139,13 @@ def get_columns_quantiles(
     :param relative_error: The relative error on the quantile approximation
     :return: Dict of column -> quantiles
     """
-
     check_ht_fields_for_spark(ht, fields)
 
     df = ht.key_by().select(*fields).to_spark()
 
     res = {}
     for f in fields:
-        logger.info("Computing median for column: {}".format(f))
+        logger.info("Computing median for column: %s", f)
         col_no_na = df.select(f).dropna()
         if col_no_na.first() is not None:
             res[f] = col_no_na.approxQuantile(str(f), quantiles, relative_error)
@@ -159,11 +155,77 @@ def get_columns_quantiles(
     return res
 
 
+def median_impute_features(
+    ht: hl.Table, strata: Optional[Dict[str, hl.expr.Expression]] = None
+) -> hl.Table:
+    """
+    Numerical features in the Table are median-imputed by Hail's `approx_median`.
+
+    If a `strata` dict is given, imputation is done based on the median of of each stratification.
+
+    The annotations that are added to the Table are
+        - feature_imputed - A row annotation indicating if each numerical feature was imputed or not.
+        - features_median - A global annotation containing the median of the numerical features. If `strata` is given,
+          this struct will also be broken down by the given strata.
+        - variants_by_strata - An additional global annotation with the variant counts by strata that will only be
+          added if imputing by a given `strata`.
+
+    :param ht: Table containing all samples and features for median imputation.
+    :param strata: Whether to impute features median by specific strata (default False).
+    :return: Feature Table imputed using approximate median values.
+    """
+    logger.info("Computing feature medians for imputation of missing numeric values")
+    numerical_features = [
+        k for k, v in ht.row.dtype.items() if v == hl.tint or v == hl.tfloat
+    ]
+
+    median_agg_expr = hl.struct(
+        **{feature: hl.agg.approx_median(ht[feature]) for feature in numerical_features}
+    )
+
+    if strata:
+        ht = ht.annotate_globals(
+            feature_medians=ht.aggregate(
+                hl.agg.group_by(hl.tuple([ht[x] for x in strata]), median_agg_expr),
+                _localize=False,
+            ),
+            variants_by_strata=ht.aggregate(
+                hl.agg.counter(hl.tuple([ht[x] for x in strata])), _localize=False
+            ),
+        )
+        feature_median_expr = ht.feature_medians[hl.tuple([ht[x] for x in strata])]
+        logger.info(
+            "Variant count by strata:\n%s",
+            "\n".join(
+                [
+                    "{}: {}".format(k, v)
+                    for k, v in hl.eval(ht.variants_by_strata).items()
+                ]
+            ),
+        )
+
+    else:
+        ht = ht.annotate_globals(
+            feature_medians=ht.aggregate(median_agg_expr, _localize=False)
+        )
+        feature_median_expr = ht.feature_medians
+
+    ht = ht.annotate(
+        **{f: hl.or_else(ht[f], feature_median_expr[f]) for f in numerical_features},
+        feature_imputed=hl.struct(
+            **{f: hl.is_missing(ht[f]) for f in numerical_features}
+        ),
+    )
+
+    return ht
+
+
 def ht_to_rf_df(
     ht: hl.Table, features: List[str], label: str, index: str = None
 ) -> pyspark.sql.DataFrame:
     """
-    Creates a Spark dataframe ready for RF from a HT.
+    Create a Spark dataframe ready for RF from a HT.
+
     Rows with any missing features are dropped.
     Missing labels are replaced with 'NA'
 
@@ -177,7 +239,6 @@ def ht_to_rf_df(
     :param index: Optional index column to keep (E.g. for joining results back at later stage)
     :return: Spark Dataframe
     """
-
     cols_to_keep = features + [label]
     if index:
         cols_to_keep.append(index)
@@ -199,7 +260,6 @@ def get_features_importance(
     :param assembler_index: index of the VectorAssembler stage
     :return: feature importance for each feature in the RF model
     """
-
     feature_names = [
         x[: -len("_indexed")] if x.endswith("_indexed") else x
         for x in rf_pipeline.stages[assembler_index].getInputCols()
@@ -210,7 +270,7 @@ def get_features_importance(
 
 def get_labels(rf_pipeline: pyspark.ml.PipelineModel) -> List[str]:
     """
-    Returns the labels from the StringIndexer stage at index 0 from an RF pipeline model
+    Return the labels from the StringIndexer stage at index 0 from an RF pipeline model.
 
     :param rf_pipeline: Input pipeline
     :return: labels
@@ -239,7 +299,6 @@ def test_model(
     :param prediction_col_name: Where to store the prediction
     :return: A list containing structs with {label, prediction, n}
     """
-
     ht = apply_rf_model(
         ht.filter(hl.is_defined(ht[label])),
         rf_model,
@@ -257,12 +316,11 @@ def test_model(
     # Print results
     df = pd.DataFrame(test_results)
     df = df.pivot(index=label, columns=prediction_col_name, values="n")
-    logger.info("Testing results:\n{}".format(pprint.pformat(df)))
+    logger.info("Testing results:\n%s", pprint.pformat(df))
     logger.info(
-        "Accuracy: {}".format(
-            sum([x.n for x in test_results if x[label] == x[prediction_col_name]])
-            / sum([x.n for x in test_results])
-        )
+        "Accuracy: %f",
+        sum([x.n for x in test_results if x[label] == x[prediction_col_name]])
+        / sum([x.n for x in test_results]),
     )
 
     return test_results
@@ -277,7 +335,7 @@ def apply_rf_model(
     prediction_col_name: str = "rf_prediction",
 ) -> hl.Table:
     """
-    Applies a Random Forest (RF) pipeline model to a Table and annotate the RF probabilities and predictions.
+    Apply a Random Forest (RF) pipeline model to a Table and annotate the RF probabilities and predictions.
 
     :param ht: Input HT
     :param rf_model: Random Forest pipeline model
@@ -287,7 +345,6 @@ def apply_rf_model(
     :param prediction_col_name: Name of the column that will store the RF predictions
     :return: Table with RF columns
     """
-
     logger.info("Applying RF model.")
 
     check_ht_fields_for_spark(ht, features + [label])
@@ -310,7 +367,8 @@ def apply_rf_model(
 
         return udf(to_array_, ArrayType(DoubleType()))(col)
 
-    # Note: SparkSession is needed to write DF to disk before converting to HT; hail currently fails without intermediate write
+    # Note: SparkSession is needed to write DF to disk before converting to HT;
+    # hail currently fails without intermediate write
     spark = SparkSession.builder.getOrCreate()
     rf_df.withColumn("probability", to_array(col("probability"))).select(
         [index_name, "probability", "predictedLabel"]
@@ -340,14 +398,14 @@ def save_model(
     rf_pipeline: pyspark.ml.PipelineModel, out_path: str, overwrite: bool = False
 ) -> None:
     """
-    Saves a Random Forest pipeline model.
+    Save a Random Forest pipeline model.
 
     :param rf_pipeline: Pipeline to save
     :param out_path: Output path
     :param overwrite: If set, will overwrite existing file(s) at output location
     :return: Nothing
     """
-    logger.info("Saving model to %s" % out_path)
+    logger.info("Saving model to %s", out_path)
     if overwrite:
         rf_pipeline.write().overwrite().save(out_path)
     else:
@@ -356,12 +414,12 @@ def save_model(
 
 def load_model(input_path: str) -> pyspark.ml.PipelineModel:
     """
-    Loads a Random Forest pipeline model.
+    Load a Random Forest pipeline model.
 
     :param input_path: Location of model to load
     :return: Random Forest pipeline model
     """
-    logger.info("Loading model from {}".format(input_path))
+    logger.info("Loading model from %s", input_path)
     return pyspark.ml.PipelineModel.load(input_path)
 
 
@@ -373,7 +431,7 @@ def train_rf(
     max_depth: int = 5,
 ) -> pyspark.ml.PipelineModel:
     """
-    Trains a Random Forest (RF) pipeline model.
+    Train a Random Forest (RF) pipeline model.
 
     :param ht: Input HT
     :param features: List of columns to be used as features
@@ -382,13 +440,12 @@ def train_rf(
     :param max_depth: Maximum tree depth
     :return: Random Forest pipeline model
     """
-
     logger.info(
-        "Training RF model using:\n"
-        "features: {}\n"
-        "labels: {}\n"
-        "num_trees: {}\n"
-        "max_depth: {}".format(",".join(features), label, num_trees, max_depth)
+        "Training RF model using:\nfeatures: %s\nlabels: %s\nnum_trees: %d\nmax_depth: %d",
+        ",".join(features),
+        label,
+        num_trees,
+        max_depth,
     )
 
     check_ht_fields_for_spark(ht, features + [label])
@@ -401,11 +458,11 @@ def train_rf(
         .fit(df)
     )
     labels = label_indexer.labels
-    logger.info("Found labels: {}".format(labels))
+    logger.info("Found labels: %s", labels)
 
     string_features = [x[0] for x in df.dtypes if x[0] != label and x[1] == "string"]
     if string_features:
-        logger.info("Indexing string features: {}".format(",".join(string_features)))
+        logger.info("Indexing string features: %s", ",".join(string_features))
     string_features_indexers = [
         StringIndexer(inputCol=x, outputCol=x + "_indexed")
         .setHandleInvalid("keep")
@@ -446,26 +503,77 @@ def train_rf(
     feature_importance = get_features_importance(rf_model)
 
     logger.info(
-        "RF features importance:\n{}".format(
-            "\n".join(["{}: {}".format(f, i) for f, i in feature_importance.items()])
-        )
+        "RF features importance:\n%s",
+        "\n".join([f"{f}: {i}" for f, i in feature_importance.items()]),
     )
 
     return rf_model
 
 
+def get_rf_runs(rf_json_fp: str) -> Dict:
+    """
+    Load RF run data from JSON file.
+
+    :param rf_json_fp: File path to rf json file.
+    :return: Dictionary containing the content of the JSON file, or an empty dictionary if the file wasn't found.
+    """
+    if file_exists(rf_json_fp):
+        with hl.hadoop_open(rf_json_fp) as f:
+            return json.load(f)
+    else:
+        logger.warning(
+            "File %s could not be found. Returning empty RF run hash dict.", rf_json_fp
+        )
+        return {}
+
+
+def get_run_data(
+    input_args: Dict[str, bool],
+    test_intervals: List[str],
+    features_importance: Dict[str, float],
+    test_results: List[hl.tstruct],
+) -> Dict:
+    """
+    Create a Dict containing information about the RF input arguments and feature importance.
+
+    :param Dict of bool keyed by str input_args: Dictionary of model input arguments
+    :param List of str test_intervals: Intervals withheld from training to be used in testing
+    :param Dict of float keyed by str features_importance: Feature importance returned by the RF
+    :param List of struct test_results: Accuracy results from applying RF model to the test intervals
+    :return: Dict of RF information
+    """
+    run_data = {
+        "input_args": input_args,
+        "features_importance": features_importance,
+        "test_intervals": test_intervals,
+    }
+
+    if test_results is not None:
+        tps = 0
+        total = 0
+        for row in test_results:
+            values = list(row.values())
+            # Note: values[0] is the TP/FP label and values[1] is the prediction
+            if values[0] == values[1]:
+                tps += values[2]
+            total += values[2]
+        run_data["test_results"] = [dict(x) for x in test_results]
+        run_data["test_accuracy"] = tps / total
+
+    return run_data
+
+
 def pretty_print_runs(
     runs: Dict, label_col: str = "rf_label", prediction_col_name: str = "rf_prediction"
 ) -> None:
     """
-    Prints the information for the RF runs loaded from the json file storing the RF run hashes -> info
+    Print the information for the RF runs loaded from the json file storing the RF run hashes -> info.
 
     :param runs: Dictionary containing JSON input loaded from RF run file
     :param label_col: Name of the RF label column
     :param prediction_col_name: Name of the RF prediction column
     :return: Nothing -- only prints information
     """
-
     for run_hash, run_data in runs.items():
         print(f"\n=== {run_hash} ===")
         testing_results = (
@@ -479,4 +587,4 @@ def pretty_print_runs(
             res_pd = res_pd.pivot(
                 index=label_col, columns=prediction_col_name, values="n"
             )
-            logger.info("Testing results:\n{}".format(pformat(res_pd)))
+            logger.info("Testing results:\n%s", pformat(res_pd))
